function [precompresion, postcompresion amp] = elab_phantom(N,compresion)
%   [PRESCOMPRESION, POSCOMPRESION, AMP] = CREAR_PHANTOM(N,COMPRESION)
%   Entradas:
%       N: Número de dispersores
%       COMPRESION: Compresion en milimetros para el phantom de
%       postcompresion.
%   Salidas:
%       PRECOMPRESION: Coordenadas [x,y,z] de los dispersores del phantom 
%           sin comprimir. Se debe asociar a cada punto del espacio la amplitud
%           de la variable AMP.
%       POSTCOMPRESION: Coordenadas [x,y,z] de los dispersores del phantom
%           comprimido. Se debe asociar a Se debe asociar a cada punto del espacio la amplitud
%           de la variable AMP.
%    
%   Autor: Daniel Serra Freese
%   Web: www.dranis.es
%   e-mail: danielserrafreese@gmail.com
%
%   Version: jueves 12 de enero, 2012.

% N = 1e3; % Valor ejemplo
% compresion = 10/1000; % Valor ejemplo

%% Parametros de visualizacion
markerSize = 5; % Tamano del marcador para scatter plot


%% Carga de imágenes
% Cargamos a partir de una imagen BMP la geometria del modelo. La imagen
% debe estar en blanco y negro. Para evitar problemas mejor editar modelo
% actual y guardar. Se pueden utilizar varias escalas de gris. Por otra
% parte, hay que llegar a un compromiso entre cantidad de dispersores y
% amplitudes, ya que un exceso de dispersores con gran amplitud no dejaran
% que se propaguen los ultrasonidos por el phantom (enseguida se refleja
% todo).

[mapa_ela, MAP]=bmpread('mapa_elastico.bmp'); % Mapa elastico
mapa_ela=mapa_ela';     % Transpone
[Nl, Ml]=size(mapa_ela);    % Dimensiones

[mapa_amp, MAP]=bmpread('mapa_amplitudes.bmp'); % Mapa amplitudes scatter
mapa_amp=mapa_amp';     % Transpone
[Ol, Pl]=size(mapa_amp);    % Dimensiones

if Nl ~= Ol || Ml ~= Pl
    error('Las dimensiones de los mapas no coinciden')
end


%% Dimensiones y resolucion del modelo
% Damos las dimensiones y su resolucion correspondiente al modelo. Además
% se incorpora un angulo theta para realizar una traslacion en el modelo de
% con un determinado angulo. Finalmente, se pasa la 'compresion' de
% milimetros a elementros (combinacion de elementos FEM de 1 m).

% Dimensiones del modelo.
x_size = 100/1000 ;    % Tamano en x-direccion [m]
dx=x_size/Nl;          % Intervalo de muestreo en x-direccion [m]

z_size = 100/1000 ;    % Tamano en z-direccion [m]
dz=z_size/Ml;          % Intervalo de muestreo en z-direccion [m]

y_size = 15/1000;      % Tamano en y-direccion [m]
                       %  
theta = 0/180*pi;      %  Rotation of the scatterers [rad]

z_start = 0/1000;      % Desfase por si fuera necesario para FIELD II

compresion=compresion/dz; % [elementros]

% Proporcionamos a dimension de elementos con 'dz'. Importante hacer este
% cambio ya que el modelo FEM trabaja con elementros (combinación elemento-metro).


%% Siembra de dispersores y cálculo de coordenadasispersores y obtener coordenadas.
% Sembramos dispersores aleatoriamente.
x = (rand(N,1)-0.5)* x_size;
z = rand(N,1)*z_size+z_start;  % Tiene en cuenta el desfase
y = (rand(N,1)-0.5)* y_size; 

%  Obtenemos los indices para referirnos a las posiciones de la imagen 
% cargada. Pasa de posicion física en metros a un índice.
xindex = round((x + 0.5*x_size)/dx + 1);   
zindex = round((z - z_start)/dz + 1);

% Comprueba que está dentro de las dimensiones del modelo. Si no lo está lo
% aprovecha para el siguiente paso.
inside = (0 < xindex)  & (xindex <= Nl) & (0 < zindex)  & (zindex <= Ml);
% Pasa de coordenadas bidimensionales para matrices a coordenadas en
% memoria
%
%                   |2 6 4|        M(2,2) = 1 
%               M = |4 1 6|          o también
%                   |7 3 9|        M(5) = 1
%
index = (xindex + (zindex-1)*Nl).*inside + 1*(1-inside);  % Indice a memoria

% Se generan amplitudes con diferente varianza. Se utilizan las amplitudes de la
% imagen cargada.
amp=mapa_amp(index);
amp=exp(mapa_amp(index)/100);
amp=amp-min(min(amp));
amp=1e6*amp/max(max(amp));
amp=amp.*randn(N,1).*inside;


% Terminamos de arreglar indices que pudieran estar fuera del rango
% (menores que cero o mayores que la dimension).
xindex(find(xindex <=0)) = 1;
xindex(find(xindex >=Nl))= Nl;
zindex(find(zindex <=0)) = 1;
zindex(find(zindex >=Ml))= Ml; 


%% Cálculo del modelo de compresion mediante FEM (elementos finitos)
% Ejemplo adaptado de 'MATLAB Codes for Finite Element Analysis - Solids
% and Structures' [www.matlabtrainings.blogfa.com].
% Se trata cada columna de la imagen (sobre el eje z) como un conjunto de
% muelles de longitud unidad (1 metro).

columnaActual=1; % Puntero

% Hasta que no se haya recorrido toda la imagen no se para.
while columnaActual <= Nl
    
    % Los nodos son antes y despues de cada elemento. Es decir, el elemento
    % 1 está formado por el nodo 1 y 2.
    %      
    %          /|    k1               Además se definen todas las fuerzas
    %          /o--\/\/\---o-- 
    %          /|n1         n2
    %
        for i=1:Ml
        elementoNodos(i,:)=[i i+1];  % Generamos los nodos
        end
    
    numeroElementos=size(elementoNodos,1);
    numeroNodos=Ml+1;   % Incluimos el nodo del final
    
    desplazamientos = zeros(numeroNodos,1);     % Prelocalizamos memoria
    fuerzas = zeros(numeroNodos,1);
        
    k = mapa_ela(columnaActual,:);  % Obtenemos elasticidades de una columna
    
    kEq = 1/sum(1./k);              % Constante elastica equivalente para la columna
    fuerzaGlobal=kEq*compresion;    % Fuerza aplicada a la linea [m] para conseguir dicha compresion

    fuerzas(Ml) = -fuerzaGlobal;    % Definimos que la fuerza se aplica en el último elemento
    rigidez=zeros(numeroNodos);
    
    
    % Calculamos la matriz de rigidez (la grande) que define a todo el sistema 
    for e=1:numeroElementos;
        % elementoDdL: Dominio de libertad del elemento (DdL)
        elementoDdL=elementoNodos(e,:);
        
        % Se mete compone la matriz de la forma q=ka (vea páginas 19 a 21 del libro)        
        rigidez(elementoDdL,elementoDdL)=rigidez(elementoDdL,elementoDdL)+k(e)*[1 -1;-1 1];
    end
    
    % Condiciones de contorno (CC)
    % Nodos bloqueados
    nodosBloqueados=[1];  % Bloqueamos el primer nodo
    % Nodos libres
    nodosLibres=setdiff([1:numeroNodos]',[nodosBloqueados]); % Los que no estan bloqueados
    
    % Solucion
    desplazamientos=rigidez(nodosLibres,nodosLibres)\fuerzas(nodosLibres); % Posicionamos todos los desplazamientos
    desplazamientos1=zeros(numeroNodos,1);
    desplazamientos1(nodosLibres)=desplazamientos;
    
    % outputDisplacementsReactions(desplazamientos1,rigidez,numeroNodos,nodosBloqueados)
    
    datosSalida(:,columnaActual)=desplazamientos1; % Guardamos los desplazamientos calculados.
    
    columnaActual = columnaActual + 1; % Seguimos con la siguiente columna

end

% Pasamos de elementros a posiciones fisicas mediante dz.
datosSalida1=datosSalida*dz;



%% Compresion del modelo
% Mediante las deformaciones calculadas procedemos a desplazar los puntos
% del modelo original.

xindex = uint16(xindex);    % No es necesario pero ayuda
zindex = uint16(zindex);    % Para asegurar que los indices son enteros

% Obtenemos para cada dispersor sus coordenadas y le asocioamos
% desplazamiento
for i=1:N
    columna = xindex(i);
    hilera = zindex(i);
    desplazamientos(i) = datosSalida1(hilera,columna);
end

% Desplazamos coordenadas
xPostcompresion = x;
yPostcompresion = y;
zPostcompresion = z + desplazamientos; 


%% Representacion de los modelos (No necesario)
% Tenga en cuenta que para representar modelos de muchos dispersores se
% puede tardar mucho en generar los gráficos.
 
% Precompresion
figure,
subplot(1,2,1)
C  = repmat([1 0 0],[N,1]); % Por si se quiere un único color: C <==> log10(amp)
scatter3(x,y,z,markerSize,log10(amp));
set(gca,'YDir','reverse')
xlabel('Eje x (m)'); ylabel('Eje y  (m)'); zlabel('Eje z (m)')
axis equal;
axis([-x_size/2 x_size/2 -y_size/2 y_size/2 z_start (z_start+z_size)])
title('Modelo de precompresion')

% Postcompresion
subplot(1,2,2)
C  = repmat([0 0 1],[N,1]);  % Por si se quiere un único color C<==>log10(amp)
scatter3(xPostcompresion,yPostcompresion,zPostcompresion,markerSize,log10(amp));
set(gca,'YDir','reverse')
xlabel('Eje x'); ylabel('Eje y'); zlabel('Eje z')
axis equal;
axis([-x_size/2 x_size/2 -y_size/2 y_size/2 z_start (z_start+z_size)])
title('Modelo de postcompresion')

drawnow

% Comparacion de modelos 2D
figure
subplot(1,2,1)
C  = repmat([1 0 0],[N,1]);
scatter(x,z,markerSize,log10(amp),'filled')
axis equal tight;
xlabel('Eje x  (m)'); ylabel('Eje z (m)');
title('Modelo de precompresion')
axis([-x_size/2 x_size/2 z_start (z_start+z_size)])

subplot(1,2,2)
C  = repmat([0 0 1],[N,1]);
scatter(xPostcompresion,zPostcompresion,markerSize,log10(amp),'filled')
axis equal tight;
xlabel('Eje x'); ylabel('Eje z');
title('Modelo de postcompresion')
axis([-x_size/2 x_size/2 z_start (z_start+z_size)])

drawnow

%% Representacion del mapa de deformacion (No necesario)

figure;
subplot(1,2,1)
pcolor(datosSalida1), axis equal tight, shading interp
title('Mapa de desplazamiento')

subplot(1,2,2)
pcolor(diff(datosSalida1)), axis equal tight, shading interp
title('Mapa de deformacin')
drawnow;

%% Datos de salida
% Devolvemos las variables de salida de la función además de guardar los
% datos por si fueran necesarios posteriormente.

precompresion = [x y z];
postcompresion = [xPostcompresion yPostcompresion zPostcompresion];

% Guardamos datos las localizaciones de los dispersores.
save phantom_data.mat x y z xPostcompresion yPostcompresion zPostcompresion

% Guardamos datos de salida de las dimensiones del phantom.
save phantom_parameters.mat N compresion x_size dx z_size dz y_size z_start

%% Añadimos datos al archivo de información

fid = fopen('info.txt', 'a');
info_00 = 'Información del modelo:';
info_01 = ['Cantidad de dispersores: ', num2str(N)];
info_02 = ['Compresion aplicada al modelo [mm]: ',num2str(compresion*1000*dz)];
info_03 = ['Largo (eje x) [mm]: ', num2str(round(x_size*1000))];
info_04 = ['Ancho (eje y) [mm]: ', num2str(y_size*1000)];
info_05 = ['Alto (eje z) [mm]: ', num2str(z_size*1000)];
info_06 = ['Incremento x [mm]: ', num2str(dx)];
info_07 = ['Incremento z [mm]: ', num2str(dz)];

fprintf(fid,info_00);
fprintf(fid,'\n\n\t');
fprintf(fid,info_01);
fprintf(fid,'\n\n\t');
fprintf(fid,info_02);
fprintf(fid,'\n\n\t');
fprintf(fid,info_03);
fprintf(fid,'\n\t');
fprintf(fid,info_04);
fprintf(fid,'\n\t');
fprintf(fid,info_05);
fprintf(fid,'\n\n\t');
fprintf(fid,info_06);
fprintf(fid,'\n\t');
fprintf(fid,info_07);
fprintf(fid,'\n\n');

fclose(fid);

end


